#########################################################################
# Copyright (c) 2002, 2003 Grgoire Weber. All Rights Reserved.
# 
# This file is part of CMFCollection.
# 
# CMFCollection is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CMFCollection is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with CMFCollection; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#########################################################################
""" XXX



$Id: $
"""
__version__ = "$Revision: $"

import sys, traceback
import string, types

import zLOG
from Globals import InitializeClass
from OFS.CopySupport import CopyError
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.CMFCore.CMFCorePermissions import ModifyPortalContent
from OrderedPortalFolder import OrderedPortalFolder
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore import utils
from Products.CMFCore.utils import getToolByName
from AccessControl import Permissions, ClassSecurityInfo, getSecurityManager, Unauthorized
from DateTime import DateTime
from Acquisition import aq_base, aq_parent, aq_inner
from ComputedAttribute import ComputedAttribute
from ExtensionClass import Base

# Import permission names
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import _checkPermission

from ObjectAnnotations import ObjectAnnotations
import Names
import FactoryDefaults

# Factory type information -- makes CMFCollection content objects play nicely
# with the Types Tool (portal_types)

def addCollectionSlot(self, id, title=''):
    """Create an empty slot.
    """

    slot = CollectionSlot(id)
    slot.id = id
    slot.title = title
    self._setObject(id, slot)

    return self._getOb(id)

class CollectionSlot(OrderedPortalFolder):
    """
    CollectionSlot holds ordered collection of content types
    text, images, tables, links etc...
    """
    try:
        __implements__ = ( PortalContent.__implements__
                         , DefaultDublinCoreImpl.__implements__
                         )
    except:
        pass

    meta_type = Names.collection_slot_meta_type

    _isDiscussable = 0

#    isPrincipiaFolderish = 0

    manage_options = ( OrderedPortalFolder.manage_options[:3] +
                       ( { 'label'  : 'Dublin Core',
                           'action' : 'manage_metadata',
                           },
                         ) +
                       OrderedPortalFolder.manage_options[3:]
                       )

    # Use Zope 2.3 declarative security
    security = ClassSecurityInfo()
    security.declareObjectProtected(CMFCorePermissions.View)


    # ---
    # ZMI
    # ---

    security.declareProtected(CMFCorePermissions.View, 'icon')
    def icon(self):
        """ For the ZMI """
        return self.getIcon()

    ### fix me: remove dummy methods to make known permissions
    def x(self): pass
    def y(self): pass
    def z(self): pass

    security.declareProtected(Names.AddContentToCollections, 'x')
    security.declareProtected(Names.ProposeContentToCollections, 'y')
    security.declareProtected(Names.ManageContentInCollections, 'z')


    # ----------
    # Cataloging
    # ----------

    def SearchableText(self):
        """SearchableText returns text visible by the collection view.
        """
        stext = [self.title, self.description]
        for o in self.objectValues():
            if self.isTitleVisible(o):
                stext.append(o.Title())
            if self.isDescriptionVisible(o):
                stext.append(o.Description())

        return ' '.join(stext)


    # --------------------------------
    # control adding objects to a slot
    # --------------------------------

    def _setObject(self, id, object, roles=None, user=None, set_owner=1):
        # call OrderedPortalFolders _setObject
        result = OrderedPortalFolder._setObject(self, id, object, roles, user, set_owner)
        object = self._getOb(result)

        # initialize the object with default values if not yet initialized
        if getattr(aq_base(object), 'collection', None) is None:
            object.collection = ObjectAnnotations()
            self._initAddedObject(object)
        else:
            object.collection.delHelpers()

        return result

    # XXX this hack seems not to be necessary (corrected wrong bug :-( )
    def xxx_manage_cutObjects(self, ids=[], REQUEST=None, is_moveonly=None):
        """Cut objects from slot.
        
        By setting 'is_moveonly' to a true value, the move operation does 
        not change the modification date of the object. 
        
        Caution: The caller has to ensure that every cutted object gets 
        pasted during the same transaction or the transaction gets aborted!
        
        This feature is used by the 'moveObjects' method of the
        CollectionType class in the Collectiontype module.
        """
        if is_moveonly:
            # override the 'notifyModified' class method by a dummy
            # instance method 'temp_notifyModified'. This way we avoid 
            # that the original 'notifyModified' method can change the 
            # modification date. 'temp_notifyModified' deletes itself
            # from the instance on being called.
            for id in ids:
                o = getattr(aq_base(self), id)
                def temp_notifyModified(o=o):
                    del o.notifyModified
                o.notifyModified = temp_notifyModified
        
        # do cutting
        return OrderedPortalFolder.manage_cutObjects(self, ids=ids, REQUEST=REQUEST)

    # --------------
    # helper methods
    # --------------

    def _getConfigurationObject(self):
        """Returns the configuration object of the collection,
        the slot in the collection or the helper in a slot.
        """
        portal_collections = getToolByName(self, 'portal_collections')
        return getattr(getattr(portal_collections, Names.types_id),
            aq_base(aq_parent(self)).portal_type)

    def getOriginal(self, obj):
        """Returns the original object for a given 'obj' or None.
        """
        portal_collections = getToolByName(self, 'portal_collections')
        originals_puid = getattr(aq_base(obj), 'originals_puid', None)
        if originals_puid is not None:
            return portal_collections.getObject(originals_puid)
        return None

    security.declarePrivate('getCopy')
    def getCopy(self, obj):
        """Returns the copy of the original object in the collection or None.
        """
        portal_collections = getToolByName(self, 'portal_collections')

        puid = portal_collections.getPuid(obj)
        impossible_puid = portal_collections.getImpossiblePuid()
        copies = filter(lambda o, p=puid, i=impossible_puid:
                        getattr(o, 'originals_puid', i) == p,
                        self.objectValues())
        if len(copies) > 1:
            zLOG.LOG("CMFCollection.CollectionType.getCopy:", zLOG.INFO,
                     "ASSERT: there is more than one copy of the original with puid %s." % puid)
        try:
            return copies[0]
        except IndexError:
            return None

    def _initAddedObject(self, obj):
        """Initialize added object.
        """
        config = self._getConfigurationObject()
        slot_config = getattr(aq_base(config), self.getId())
        anno = obj.collection

        # set initial position
        initial_position = slot_config.getProperty('initial_position')
        if initial_position == 'top':
            self.move_objects_to_top(obj.getId())
        else: # initial_position == 'bottom'
            self.move_objects_to_bottom(obj.getId())

        # set visibility
        anno.setTitleVisibility(
            slot_config.getProperty('default_title_visibility'))
        anno.setDescriptionVisibility(
            slot_config.getProperty('default_description_visibility'))
        anno.setBodyVisibility(
            slot_config.getProperty('default_body_visibility'))
        
        # set default helper templates
        for helper_obj in slot_config.objectValues():
            default_template = helper_obj.getProperty('default_template')
            anno.setTemplate(helper_obj.getId(), default_template)

    security.declarePrivate('addOrUpdateObject')
    def addOrUpdateObject(self, action, orig_obj, copy_obj, status):
        """Add an object to the Slot.

           Users adding an object must have the 'AddContentToCollections'
           or 'ProposeContentToCollections' permission.
        """
        if action != 'nop':
            if action == 'add':
                # generate new id if there is an object with the same id already
                id = orig_id = orig_obj.getId()
                count = 2
                while 1:
                    if self._getOb(id, None) is None:
                        break
                    id = '%s%s' % (orig_id, count)
                    count += 1

            elif action == 'update':
                # save annotated information of object before deletion of object
                id = copy_obj.getId()
                saved_anno = copy_obj.collection.getAnnotation()
                saved_pos = self.get_object_position(id)

                # then delete the copy in the collection
                self.manage_delObjects(id)

            # get a copy (the follwing code is 'borrowed' from the 
            # manage_pasteObjects method of the OFS/CopySupport.py module)
            if not (hasattr(orig_obj, '_canCopy') and orig_obj._canCopy(0)):
                raise CopyError, "Adding '%s' to collections is not supported." % orig_obj.getId()
            copy_obj = orig_obj._getCopy(self)
            copy_obj._setId(id)
            self._setObject(id, copy_obj)
            copy_obj = self._getOb(id)

            # set the originals puid information in the copied object
            portal_collections = getToolByName(self, 'portal_collections')
            copy_obj.originals_puid = portal_collections.getPuid(orig_obj)

            # avoid duplicate puids: delete the copies puid and
            # give it a new one (getPuid sets a new one if none available)
            portal_collections.delPuid(copy_obj)
            portal_collections.getPuid(copy_obj)

            # adding a manual collection means adding a proxy
            # collection (an empty one)
            if copy_obj.meta_type == Names.collection_meta_type:
                copy_obj.manage_delObjects(copy_obj.getObjectIds())

            if action == 'add':
                # initialize collection specific parts of object
                self._initAddedObject(copy_obj)

            elif action == 'update':
                # recall annotated information
                copy_obj.collection.setAnnotation(saved_anno)
                # move back to the previous position
                self.move_objects_by_positions(id, saved_pos-self.get_object_position(id))

        # manage status
        copy_obj.collection.setStatus(status)

    def _absattr(self, attr):
        if callable(attr): return attr()
        return attr

    def _isMoveable(self, obj):
        # Is object moveable? Returns 0 or 1 (the follwing code 
        # is 'borrowed' from the manage_pasteObjects method of the 
        # OFS/CopySupport.py module)
        if not (hasattr(obj, '_canCopy') and obj._canCopy(1)):
            return 0
        if hasattr(obj, '_p_jar') and obj._p_jar is None:
            return 0
        try:    n=aq_parent(aq_inner(obj))._reserved_names
        except: n=()
        if self._absattr(obj.id) in n:
            return 0
        return 1

    # XXX: have a look at cataloging when doing add or move ether by
    #      using manage_* or doing things by hand (as done in CopySupport)

    security.declarePrivate('xxx_moveObject')
    def xxx_moveObject(self, obj):
        """moves object from another slot to this slot
        """
        # the follwing code is 'borrowed' from the manage_pasteObjects 
        # method of the OFS/CopySupport.py module
        id = obj.getId()
        if not self._isMoveable(obj):
            raise CopyError, "Moving '%s' from one slot to " \
                             "another is not supported." % id
        
        # try to make ownership explicit so that it gets carried
        # along to the new location if needed.
        # XXX doesn't work in unit tests and I don't understand 
        #     exactly why it necessary in our case
#        obj.manage_changeOwnershipType(explicit=1)

        # delete object from source slot
        aq_parent(aq_inner(obj))._delObject(id)
        obj = aq_base(obj)

        # generate new id if there is an object with the same id already
        id = orig_id = obj.getId()
        count = 2
        while 1:
            if self._getOb(id, None) is None:
                break
            id = '%s%s' % (orig_id, count)
            count += 1

        # set object at destination slot
        obj._setId(id)
        self._setObject(id, obj, set_owner=0)

        # try to make ownership implicit if possible
        obj=self._getOb(id)

        # XXX doesn't work in unit tests and don't understand 
        #     exactly if necessar in our case
#        obj.manage_changeOwnershipType(explicit=0)

        return id

    # --------------
    # object setters
    # --------------

    security.declarePrivate('setTitleVisibility')
    def setTitleVisibility(self, obj, visibility):
        """Sets the titles visibility.
        """
        obj.collection.setTitleVisibility(visibility)

    security.declarePrivate('setDescriptionVisibility')
    def setDescriptionVisibility(self, obj, visibility):
        """Sets the descriptions visibility.
        """
        obj.collection.setDescriptionVisibility(visibility)

    security.declarePrivate('setBodyVisibility')
    def setBodyVisibility(self, obj, visibility):
        """Sets the bodies visibility.
        """
        obj.collection.setBodyVisibility(visibility)

    security.declarePrivate('setSelectedTemplate')
    def setSelectedTemplate(self, obj, helper, template):
        """Sets the objects helper to template.
        """
        obj.collection.setTemplate(helper, template)

    security.declarePrivate('setSelectedTemplate')
    def moveRelative(self, id, positions):
        """Moves one object by positions (relative mode).
        """
        self.move_objects_by_positions(id, positions)

    security.declarePrivate('setSelectedTemplate')
    def moveAbsolute(self, id, position):
        """Moves one object to position (absolute mode).
        """
        if position < 0:
            self.move_objects_by_positions(id, len(self.objectIds())+position-self.get_object_position(id))
        else:
            self.move_objects_by_positions(id, position-self.get_object_position(id))

    # -------
    # web API
    # -------

    security.declareProtected(CMFCorePermissions.View, 'listSlotContents')
    def listSlotContents(self):
        return [o for o in self.listFolderContents() if self.isVisible(o)]

    security.declareProtected(CMFCorePermissions.View, 'getSlotDefinitionProperty')
    def getSlotDefinitionProperty(self, name):
        """Returns a property value from the slot definition.

        Use this method to access custom definition properties.
        Use specialized methods 'getTitle', 'getDescription' and
        'getDefaultText' for predefined properties.
        """
        portal_collections = getToolByName(self, 'portal_collections')
        return getattr(getattr(portal_collections, Names.slots_id),
            self.getId()).getProperty(name)

    security.declareProtected(CMFCorePermissions.View, 'getTitle')
    def getTitle(self):
        """Returns the slots title.

        The title returned is the one defined in the slot definition.
        """
        return self.getSlotDefinitionProperty('title')

    security.declareProtected(CMFCorePermissions.View, 'getDescription')
    def getDescription(self):
        """Returns the slots description.

        The description returned is the one defined in the slot
        definition.
        """
        return self.getSlotDefinitionProperty('description')

    security.declareProtected(CMFCorePermissions.View, 'getDefaultText')
    def getDefaultText(self):
        """Returns the slots default text in case of an empty slot.

        The default text returned is the one defined in the slot
        definition. 
        """
        return self.getSlotDefinitionProperty('default_text')

    security.declareProtected(CMFCorePermissions.View, 'getPos')
    def getPos(self, obj):
        """Returns position of 'obj'.
        """
        return self.get_object_position(obj.getId())

    security.declareProtected(CMFCorePermissions.View, 'isTitleVisible')
    def isTitleVisible(self, obj):
        """Returns true value if objects title is visible.
        """
        return obj.collection.isTitleVisible()

    security.declareProtected(CMFCorePermissions.View, 'isTitleLevel')
    def isTitleLevel(self, obj, level):
        """Returns true value if title is visible and <hx> level is 'level'.
        """
        return obj.collection.isTitleVisible() and \
               (aq_parent(self).isTitleVisible() and 1 or 0) == level - 1

    security.declareProtected(CMFCorePermissions.View, 'isDescriptionVisible')
    def isDescriptionVisible(self, obj):
        """Returns true value if description is visible.
        """
        return obj.collection.isDescriptionVisible()

    security.declareProtected(CMFCorePermissions.View, 'isBodyVisible')
    def isBodyVisible(self, obj):
        """Returns true value if body is visible.
        """
        return obj.collection.isBodyVisible()

    def _getBodiesHeaderLevel(self, obj):
        """Returns the bodies <hx> level.
        """
        return 1 + (aq_parent(self).isTitleVisible() and 1 or 0) \
                 + (obj.collection.isTitleVisible() and 1 or 0)

    security.declareProtected(CMFCorePermissions.View, 'isVisible')
    def isVisible(self, obj):
        """Returns true value if there is visible content of an object.
        """
        return obj.collection.isVisible() \
               or self._areSelectedTemplatesVisible(obj)

    security.declareProtected(CMFCorePermissions.View, 'getStatus')
    def getStatus(self, obj):
        """Returns objects status.
        """
        return obj.collection.getStatus()

    security.declareProtected(CMFCorePermissions.View, 'getStatusAndActions')
    def getStatusAndActions(self, obj):
        """Returns status and actions where spaces converted to underscores.
        """
        sp2un = lambda str: '_'.join(str.split(' '))
        status = obj.collection.getStatus()
        accept, reject = obj.collection.getActions()
        result = {}
        result['status'] = {'name': status, 'var': sp2un(status)}
        result['actions'] = accept is not None or reject is not None
        result['accept'] = accept and {'name': accept, 'var': sp2un(accept)}
        result['reject'] = reject and {'name': reject, 'var': sp2un(reject)}

        return result

    security.declareProtected(CMFCorePermissions.View, 'getAvailableTemplates')
    def getAvailableTemplates(self, helper_id):
        """Returns available templates for this objects helper.
        """
        config = self._getConfigurationObject()
        slot = getattr(config, self.getId())
        helper = getattr(slot, helper_id)
        template_ids = helper.getProperty('allowed_templates')
        portal_collections = getToolByName(self, 'portal_collections')
        helper_def = getattr(aq_base(portal_collections), Names.helpers_id)
        helper_cfg = getattr(aq_base(helper_def), helper_id)
        template_titles = [t.title_or_id() for t in helper_cfg.objectValues()]
        templates = [{'id': t, 
                      'title': getattr(aq_base(helper_cfg), t).title_or_id()}
                     for t in template_ids]
        return templates
    
    security.declareProtected(CMFCorePermissions.View, 'getSelectedTemplateId')
    def getSelectedTemplateId(self, obj, helper_id):
        """Returns the objects helper template.
        """
        try:
            template_id = obj.collection.getTemplate(helper_id)
            if template_id:
                return template_id
        except KeyError:
            pass
        
        config = self._getConfigurationObject()
        slot = getattr(config, self.getId())
        helper = getattr(slot, helper_id)
        template_id = helper.getProperty('default_template')
        
        return template_id or helper.getProperty('allowed_templates')[0]
    
    def _areSelectedTemplatesVisible(self, obj):
        # returns a true value if at least one of the selected helper
        # templates is visible
        portal_collections = getToolByName(self, 'portal_collections')
        helper_def = aq_base(getattr(aq_base(portal_collections), Names.helpers_id))
        visible = 0
        
        for helper_ref in self.getHelperValues():
            helper_id = helper_ref.getId()
            helper_cfg = getattr(helper_def, helper_id)
            template_id = self.getSelectedTemplateId(obj, helper_id)
            template_cfg = getattr(helper_cfg, template_id)
            visible |= template_cfg.getProperty('is_visible_info')
            if visible: break
            
        return visible

    security.declareProtected(CMFCorePermissions.View, 'isSelectedTemplate')
    def isSelectedTemplate(self, obj, helper, template):
        """Returns true if the helpers template is the selected one.
        """
        return self.getSelectedTemplateId(obj, helper) == template

    security.declareProtected(CMFCorePermissions.View, 'getHelperValues')
    def getHelperValues(self, choosable_only=0):
        """Returns available helpers as objects.

        With 'choosable_only' set to a true value this method
        returns helpers only where the user can choose from
        different templates.
        """
        config = self._getConfigurationObject()
        slot = getattr(config, self.getId())
        helpers = slot.objectValues()

        if choosable_only:
            return filter(lambda f:len(f.getProperty('allowed_templates'))>1, helpers)
        else:
            return helpers

    security.declareProtected(CMFCorePermissions.View, 'getLinkedObject')
    def getLinkedObject(self, coll_obj, extended_info=None, major_only=None):
        """Returns the reference to the original or copy depending
           on the choosen policy.
        """
        # if the collection object is a collection also, always return the
        # original (as major reference and no minor reference)
        config = self._getConfigurationObject()
        if coll_obj.portal_type == Names.collection_meta_type:
            # link policy for collections is always 'OriginalAlways'
            link_policy = Names.OriginalAlways
            # do not return minor information
            major_only = 1
        else:
            # the resulting object reference depends heavily on the link policy
            link_policy = config.getProperty('link_policy')
        
        track_changes = config.getProperty('track_changes') and \
                        self.getStatus(coll_obj) in Names.UpToDateStatus
        
        # prepare modification and deletion information
        orig_obj = None
        deleted = 0
        modified = 0
        
        # We try to not get the original object if not necessary.
        # This is the case if no extended info is required and the
        # policy is CopyAlways and changes should not be tracked for.
        if extended_info or link_policy != Names.CopyAlways or track_changes:
            orig_obj = self.getOriginal(coll_obj)
            if orig_obj is None:
                deleted = 1
            else:
                modified = (orig_obj.modified()!= coll_obj.modified())
        
        # calculate major and minor object references depending on the 
        # link policy. The major object is the one that gets linked to.
        if link_policy == Names.OriginalAlways:
            major_obj = orig_obj or coll_obj
            minor_obj = orig_obj and coll_obj
        elif link_policy == Names.OriginalIfIdenticalCopyElse:
            major_obj = (modified or deleted) and coll_obj or orig_obj
            minor_obj = modified and orig_obj or not deleted and coll_obj or None
        elif link_policy == Names.CopyIfIdenticalOriginalElse:
            major_obj = (not modified or deleted) and coll_obj or orig_obj
            minor_obj = not modified and orig_obj or not deleted and coll_obj or None
        else: # link_policy == Names.CopyAlways:
            major_obj = coll_obj
            minor_obj = orig_obj
        
        # calculate status
        status = None
        if track_changes:
            status = modified and Names.ChangeNotificationPending or \
                     deleted and Names.DeleteNotificationPending or None
        
        # Has the collection object to be updated?
        # This is the case if auto_update is set and the configured 
        # link policy belong to the trusted ones and the original object 
        # is modified
        if modified and link_policy in Names.TrustedLinkPolicies and \
           config.getProperty('auto_update'):
            self.addOrUpdateObject('update', orig_obj, coll_obj, 
                                   status or Names.Updated)
        elif status is not None:
            coll_obj.collection.setStatus(status)
        
        if extended_info:
            # return extended values
            if major_only:
                minor_obj = None
            major_id = (major_obj is orig_obj) and 'Original' or 'Copy'
            minor_id = (minor_obj is coll_obj) and 'Copy' or orig_obj and 'Original'
            return {'major':major_obj, 'minor':minor_obj,
                    'major_id':major_id, 'minor_id':minor_id}
        
        # return major object reference only
        return major_obj
        
    security.declareProtected(CMFCorePermissions.View, 'xxx_getLinkedObject')
    def xxx_getLinkedObject(self, collection_obj, both=None):
        """Returns the reference to the original and copy depending
           on the choosen policy.
        """
        # get link policy
        config = self._getConfigurationObject()
        link_policy = config.getProperty('link_policy')

        same = self.getStatus(collection_obj) in Names.OrigCopySameStates

        # calculate link object
        # ask for original object only if necessary
        if link_policy == Names.OriginalAlways:
            major_obj = None
        elif link_policy == Names.CopyAlways:
            major_obj = collection_obj
        elif link_policy == Names.CopyIfIdenticalOriginalElse:
            major_obj = same and collection_obj or None
        elif link_policy == Names.OriginalIfIdenticalCopyElse:
            major_obj = not same and collection_obj or None
        else: # Names.CopyAlways:
            major_obj = collection_obj
            config._updateProperty('link_policy', Names.CopyAlways)
            zLOG.LOG("CMFCollection.CollectionType.getLinkedObject:", zLOG.INFO,
                     "ASSERT: unknown link policy, appied '%s'." % Names.CopyAlways)

        if major_obj is None:
            # we now need definitely to ask for the original 
            # object (there is a catalog search behind this call)
            orig_obj = major_obj = self.getOriginal(collection_obj)
        else:
            orig_obj = None

        # track original object for changes only if it seems to be up to date
        if config.getProperty('track_changes') and same:
            # get the originals object if not yet done (costy)
            if orig_obj is None:
                orig_obj = self.getOriginal(collection_obj)

            # check if deleted
            if orig_obj is None:
                collection_obj.collection.setStatus(Names.DeleteNotificationPending)

            # identity check by modification time
            elif orig_obj.modified() != collection_obj.modified():
                # reevaluate link policy
                if link_policy == Names.CopyIfIdenticalOriginalElse:
                    major_obj = orig_obj
                elif link_policy == Names.OriginalIfIdenticalCopyElse:
                    major_obj = collection_obj

                # do auto update if trusted policy and if auto
                # update configured
                if (link_policy in Names.TrustedLinkPolicies) \
                   and config.getProperty('auto_update'):
                    # update collection object (implicitely updates status)
                    self.addOrUpdateObject('update', orig_obj, collection_obj, Names.Updated)
                else:
                    # update collections objects status only
                    collection_obj.collection.setStatus(Names.ChangeNotificationPending)

        # Mode with both set to a true value returns the oroginal and 
        # the copy in the collection. The major one is the one that 
        # represents result according the configured policy.
        if both:
            if orig_obj is None:
                orig_obj = self.getOriginal(collection_obj)
            if orig_obj == major_obj:
                return {'major': orig_obj, 'minor': collection_obj, 
                        'major_id': 'Original', 'minor_id': 'Copy'}
            else:
                return {'major': collection_obj, 'minor': orig_obj, 
                        'major_id': 'Copy', 'minor_id': 'Original'}

        return major_obj

    security.declareProtected(CMFCorePermissions.View, 'getRenderedHelper')
    def getRenderedHelper(self, obj, helper_ids=None):
        """Returns rendered helper template(s).

        modes:

           1. calling self.getRenderedHelper(obj) returns a
              dictionary of the form <helper_id>:<rendered_code>.

           2. calling self.getRenderedHelper(obj, ('Skip', 'Bar',))
              returns a dictionary of the form
              <helper_id>:<rendered_code> with 'Skip' and 'Bar'
              as 'helper_ids'.

           3. calling self.getRenderedHelper(obj,'Default') returns
              the rendered template code of helper 'Default' of the
              slot the object is atteched to.
        """
        ### fix me: implementation variant: do macros perform better?

        # build a list with configured helpers
        if helper_ids is None:
            config = self._getConfigurationObject()
            helper_ids = getattr(config, self.getId()).objectIds()
            return_dict = 1
        elif type(helper_ids) == types.StringType:
            helper_ids = (helper_ids,)
            return_dict = 0
        else:
            return_dict = 1

        # traverse to the helper definition directory
        portal_collections = aq_base(getToolByName(self, 'portal_collections'))
        helpers = getattr(portal_collections, Names.helpers_id)

        # build dict <helper_id>:<rendered_code>
        dict = {}
        for helper_id in helper_ids:
            # get the id of the helper template
            template_id = self.getSelectedTemplateId(obj, helper_id)
            if not template_id:
                raise KeyError, "No template for helper " \
                                "'%s' available." % helper_id

            # traverse to the template to get the methods id    
            helper = getattr(helpers, helper_id)
            template = getattr(helper, template_id)
            template_methid = template.getProperty('template_id')

            # traverse the ZODB
            template_obj = getattr(self, template_methid, None)

            # call (render) the template object
            if template_obj is not None:
                # call the template
                dict[helper_id] = template_obj(container=self, obj=obj)
            else:
                dict[helper_id] = ''

        if return_dict:
            return dict
        else:
            return dict.values()[0]

    security.declareProtected(CMFCorePermissions.View, 'getRenderedBody')
    def getRenderedBody(self, obj, consider_level=None):
        """Returns the body of a content object or an empty string.
        
        Since collections do not know anything about other content
        types they ask a skin object to do it for them. The skin
        object is called with a one parameter named 'obj'.
        """
        # get collection types configuration object
        config = self._getConfigurationObject()
        body_generating_id = config.getProperty('body_prefix')+obj.portal_type
        body_generating_obj = getattr(self, body_generating_id, None)
        if body_generating_obj is not None:
            if consider_level:
                return body_generating_obj(obj=obj, 
                    header_level=self._getBodiesHeaderLevel(obj),
                    setlevel=1)
            else:
                return body_generating_obj(obj=obj)
        else:
            return ''

    security.declareProtected(ModifyPortalContent, 'acceptPendingRequest')
    def acceptPendingRequest(self, copy_obj):
        orig_obj = self.getOriginal(copy_obj)

        if orig_obj is None:
            # original doesn't exist anymore, so set status
            # to UpToDate
            copy_obj.collection.setStatus(Names.IsLocal)
        else:
            # original exists, so do update
            self.addOrUpdateObject('update', orig_obj, copy_obj, Names.UpToDate)

    security.declareProtected(ModifyPortalContent, 'rejectPendingRequest')
    def rejectPendingRequest(self, copy_obj):
        if copy_obj.collection.getStatus() == Names.AddRequestPending:
            # rejecting a add request means deleting the object
            self.manage_delObjects(ids=[copy_obj.getId()])
        else:
            # set status to OutOfDate
            copy_obj.collection.setStatus(Names.OutOfDate)


InitializeClass(CollectionSlot)
