#########################################################################
# Copyright (c) 2002, 2003 Grgoire Weber. All Rights Reserved.
# 
# This file is part of CMFCollection.
# 
# CMFCollection is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CMFCollection is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with CMFCollection; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#########################################################################
""" XXX



$Id: $
"""
__version__ = "$Revision: $"

import sys, traceback
import string
from types import TupleType, ListType, StringType, UnicodeType

import zLOG
from OFS.CopySupport import CopyError, cookie_path, _cb_encode, _cb_decode
from Globals import InitializeClass
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.CMFCore.CMFCorePermissions import ModifyPortalContent
from Products.CMFCore.PortalFolder import PortalFolder
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore import utils
from Products.CMFCore.utils import getToolByName
from AccessControl import Permissions, ClassSecurityInfo, getSecurityManager, Unauthorized
from DateTime import DateTime
from Acquisition import aq_base, aq_parent, aq_inner
from ComputedAttribute import ComputedAttribute

# Import permission names
from Products.CMFCore import CMFCorePermissions

from ObjectAnnotations import ObjectAnnotations
from CollectionSlot import CollectionSlot
from CollectionSlot import addCollectionSlot

import Names
import FactoryDefaults

# Factory type information -- makes CMFCollection content objects play nicely
# with the Types Tool (portal_types)

factory_type_information = ( { 'id'             : 'Collection'
                             , 'meta_type'      : 'Collection'
                             , 'title'          : 'Collection'
                             , 'description'    : 'A Collection manages ordered presentation of content.'
                             , 'icon'           : 'collection_icon.gif'
                             , 'product'        : 'CMFCollection'
                             , 'factory'        : 'addCollectionType'
                             , 'filter_content_types' : 0
                             , 'immediate_view' : 'collection_contents'
                             , 'slots'          : 'Sub-Collections, Main'
                             , 'actions'        :
                                ( { 'id'            : 'collection_contents'
                                  , 'name'          : 'Collection Contents'
                                  , 'action'        : 'collection_contents'
                                  , 'permissions'   :
                                     (CMFCorePermissions.ManageProperties,)
                                  , 'category'      : 'object'
                                  }
                                , { 'id'            : 'view' 
                                  , 'name'          : 'View'
                                  , 'action'        : 'collection_view'
                                  , 'permissions'   :
                                     (CMFCorePermissions.View,)
                                  , 'category'      : 'object'
                                  }
                                , { 'id'            : 'edit'
                                  , 'name'          : 'Edit'
                                  , 'action'        : 'collection_edit_form'
                                  , 'permissions'   :
                                     (CMFCorePermissions.ManageProperties,)
                                  , 'category'      : 'object'
                                  }
                                , { 'id'            : 'local_roles'
                                  , 'name'          : 'Local Roles'
                                  , 'action'        : 'folder_localrole_form'
                                  , 'permissions'   :
                                     (CMFCorePermissions.ManageProperties,)
                                  , 'category'      : 'object'
                                  }
                                )
                             }
                           ,
                           )

def addCollectionType(self, id, title='', REQUEST=None):
    """
    Create an empty CollectionType.
    """

    collection = CollectionType(id)
    collection.id = id
    collection.title = title
    self._setObject(id, collection)

    collection = self._getOb(id)


    if REQUEST is not None:
        REQUEST['RESPONSE'].redirect( 'manage_main' )

class CollectionType(PortalFolder, PortalContent, DefaultDublinCoreImpl):
    """
    CollectionTypes are collection of elements
    containing text, images, tables, links etc...
    """
    try:
        __implements__ = ( PortalContent.__implements__
                         , DefaultDublinCoreImpl.__implements__
                         )
    except:
        pass

    meta_type = Names.collection_meta_type

    _isDiscussable = 1

    slots = []

    manage_options = ( PortalFolder.manage_options[:3] +
                       ( { 'label'  : 'Dublin Core',
                           'action' : 'manage_metadata',
                           },
                         ) +
                       PortalFolder.manage_options[3:]
                       )

    _properties = PortalFolder._properties + \
        ({'id':'left_slots_before', 'type':'lines', 'mode':'w'}, 
         {'id':'left_slots', 'type':'lines', 'mode':''}, 
         {'id':'left_slots_after', 'type':'lines', 'mode':'w'}, 
         {'id':'right_slots_before', 'type':'lines', 'mode':'w'}, 
         {'id':'right_slots', 'type':'lines', 'mode':''}, 
         {'id':'right_slots_after', 'type':'lines', 'mode':'w'}, )

    def getProperty(self, id, d=None):
        """Override 'PropertyManager's getProperty method to allow
           dynamically generated properties. Caution: For read-only
           properties only!"""
        if self.hasProperty(id):
            prop = getattr(self, id)
            if callable(prop):
                return prop()
            else:
                return prop
        return d

    left_slots_before = ('Parent',)
    left_slots_after = ('None',)
    def left_slots(self):
        return self._build_slots('left')

    right_slots_before = ('None',)
    right_slots_after = ('None',)
    def right_slots(self):
        return self._build_slots('right')

    def _build_slots(self, attr):
        """build slots by putting slots before and after the
        slots generated by the collection.
        """
        slots_before = self.getProperty('%s_slots_before' % attr)
        if len(slots_before) == 1:
            if slots_before[0] == 'Parent':
                slots_before = getattr(aq_parent(self), '%s_slots' % attr)
            elif slots_before[0] == 'None':
                slots_before = ()
        if callable(slots_before):
            slots_before = slots_before()

        slots_after = self.getProperty('%s_slots_after' % attr)
        if len(slots_after) == 1:
            if slots_after[0] == 'Parent':
                slots_after = getattr(aq_parent(self), '%s_slots' % attr)
            elif slots_after[0] == 'None':
                slots_after = ()
        if callable(slots_after):
            slots_after = slots_after()

        if self.getSlots(visible_only=1, groups_only=(attr,)):
            coll_slots = ('here/collection_slots/macros/%sBoxes' % attr,)
        else:
            coll_slots = ()

        return tuple(slots_before) + coll_slots + tuple(slots_after)

    
    # Use Zope 2.3 declarative security
    security = ClassSecurityInfo()
    security.declareObjectProtected(CMFCorePermissions.View)

    def __init__( self
                , id
                , title=''
                , subject=()
                , description=''
                , contributors=()
                , effective_date=None
                , expiration_date=None
                , format='text/html'
                , language='de'
                , rights=''
                ):
        DefaultDublinCoreImpl.__init__( self, title, subject, description
                               , contributors, effective_date, expiration_date
                               , format, language, rights )

        self.setTitleVisibility(1)
        self.setDescriptionVisibility(1)

    def _prepareIds(self, ids=[], new_ids=None, action=None):
        """Returns list of slot objects with containing object ids.
        
        Raises exceptions if slots or objects do not exist.
        """
        if type(ids) is type(''):
            ids = (ids,)
        result = {}
        if new_ids is not None:
            new_ids = new_ids[:]
        
        for id in ids:
            try:
                slot_id, obj_id = id.split('/')
            except ValueError:
                raise 'IdError', "unable to %s objects in slot '%s'" % (action, id)
            
            if getattr(aq_base(self), slot_id, None) is None:
                raise 'AttributeError', "unable to %s objects " \
                    "in not existent slot '%s'" % (action, slot_id)
            
            if not result.has_key(slot_id):
                result[slot_id] = {
                    'ref': getattr(self, slot_id),
                    'ids': [],
                    }
                if new_ids is not None:
                    result[slot_id]['new_ids'] = []
            
            result[slot_id]['ids'].append(obj_id)
            if new_ids is not None:
                result[slot_id]['new_ids'].append(new_ids.pop(0))
        
        return result

    def manage_delObjects(self, ids=[], REQUEST=None):
        """Delete objects in different slots.
        """
        for slot in self._prepareIds(ids, action='delete').values():
            slot['ref'].manage_delObjects(slot['ids'])
        
        if REQUEST is not None:
            return self.manage_main(self, REQUEST, update_menu=1)

    def manage_renameObjects(self, ids=[], new_ids=[], REQUEST=None):
        """Rename objects in different slots.
        """
        for slot in self._prepareIds(ids, new_ids=new_ids, action='rename').values():
            slot['ref'].manage_renameObjects(slot['ids'], slot['new_ids'])
        
        if REQUEST is not None:
            return self.manage_main(self, REQUEST, update_menu=1)
        return None

    def manage_cutObjects(self, ids=[], REQUEST=None):
        """Cut objects from different slots.
        """
        clipboard = []
        cp_or_mv = 1
        for slot in self._prepareIds(ids, action='cut').values():
            cp = _cb_decode(slot['ref'].manage_cutObjects(slot['ids']))
            clipboard.extend(cp[1])
            cp_or_mv = cp[0]
        
        cp = _cb_encode((cp_or_mv, clipboard))
        
        if REQUEST is not None:
            resp=REQUEST['RESPONSE']
            resp.setCookie('__cp', cp, path='%s' % cookie_path(REQUEST))
            REQUEST['__cp'] = cp
            return self.manage_main(self, REQUEST)
        return cp

    def manage_copyObjects(self, ids=[], REQUEST=None, RESPONSE=None):
        """Copy objects from different slots.
        """
        clipboard = []
        cp_or_mv = 0
        for slot in self._prepareIds(ids, action='copy').values():
            cp = _cb_decode(slot['ref'].manage_copyObjects(slot['ids']))
            clipboard.extend(cp[1])
            cp_or_mv = cp[0]
        
        cp = _cb_encode((cp_or_mv, clipboard))
        
        if REQUEST is not None:
            resp=REQUEST['RESPONSE']
            resp.setCookie('__cp', cp, path='%s' % cookie_path(REQUEST))
            REQUEST['__cp'] = cp
            return self.manage_main(self, REQUEST)
        return cp

    def _pasteObjects(self, slot_id, cb_copy_data=None, REQUEST=None):
        """Paste object to slot.
        """
        if getattr(aq_base(self), slot_id, None) is None:
            raise CopyError, "Target slot '%s' does not exist" % slot_id
        slot_ref = getattr(self, slot_id)
        return slot_ref.manage_pasteObjects(cb_copy_data=cb_copy_data)

    def manage_pasteObjects(self, cb_copy_data=None, REQUEST=None):
        """Paste objects to 'initial_slot_added'.
        """
        slot_ref = self._getInitialAddSlotsObject()
        return self._pasteObjects(slot_ref.getId(), cb_copy_data, REQUEST)

    def splitId(self, id):
        """Split id of form 'slot/obj' into a dict {'slot': slot, 'obj': obj}
        
        Returns None if id is not of the above format.
        """
        try:
            slot, obj = id.split('/')
        except ValueError:
            return None
        return {'slot': slot, 'obj': obj}

    # --------------
    # helper methods
    # --------------

    def _getConfigurationObject(self):
        """Returns configuration object.
        """
        portal_collections = getToolByName(self, 'portal_collections')
        types_obj = getattr(aq_base(portal_collections), Names.types_id)
        return getattr(types_obj, self.portal_type)

    def _getConfigurationProperty(self, name):
        """Returns the value of a configuration property.
        """
        return self._getConfigurationObject().getProperty(name)

    def _getInitialAddSlotsObject(self):
        """Returns initial add slots object.
        """
        name = self._getConfigurationProperty('initial_slot_added')
        if name not in (None, ''):
            return self._getSlot(name)
        
        # set first slot as initial_slot_added if none defined
        name = self.getSlots()[0].getId()
        config = self._getConfigurationObject()
        config._updateProperty('initial_slot_added', name)
        
        return self._getSlot(name)

    def _getInitialProposedSlotsObject(self):
        """Returns initial update slots object.
        """
        name = self._getConfigurationProperty('initial_slot_proposed')
        if name not in (None, ''):
            return self._getSlot(name)
        
        # set first slot as initial_slot_proposed if none defined
        name = self.getSlots()[0].getId()
        config = self._getConfigurationObject()
        config._updateProperty('initial_slot_proposed', name)
        
        return self._getSlot(name)

    def _getSlot(self, name):
        # Slots get added to the collection by accessing them.
        # This is because there is no hook in portal_types (CMF 1.3)
        # which calls us (the collection) after having been added.

        # return slot object if already exists in collection
        if getattr(aq_base(self), name, None):
            return getattr(self, name)

        # check configuration if there should be a slot with 'name'
        portal_collections = getToolByName(self, 'portal_collections')
        config = getattr(getattr(portal_collections, Names.types_id), self.portal_type)
        if getattr(aq_base(config), name, None):
            # add an empty slot
            return addCollectionSlot(self, name)
        else:
            raise AttributeError, "Slot '%s' does not exist" % name

    def _getCopyAndSlot(self, obj):
        portal_collections = getToolByName(self, 'portal_collections')

        # Objects without puid impossibly can have a copy in a collection
        if portal_collections.PUID(obj) is None:
            return None, None

        # search the slots about a copy
        ### fix me: catalog is possibly faster
        for slot_obj in self.getSlots():
            copy_obj = slot_obj.getCopy(obj)
            if copy_obj:
                return copy_obj, slot_obj
                
        return None, None

    def _getInstructionsFor(self, obj):
        """Returns instructions what to do with the object.
           
        It gets either added or updated. The instructions 
        depend on how the collection type is configured 
        (chosen policies etc).
        """
        # find out if object has to be added or updated
        copy_obj, slot_obj = self._getCopyAndSlot(obj)

        # get collection types configuration object
        config = self._getConfigurationObject()

        verbose_status = config.getProperty('verbose_status')

        # user has add permission
        if utils._checkPermission(Names.AddContentToCollections, self):
            if copy_obj:
                # handle updating
                action = 'update'
                status = verbose_status and Names.Updated or Names.UpToDate
            else:
                # handle adding
                slot_obj = self._getInitialAddSlotsObject()
                action = 'add'
                status = verbose_status and Names.Added or Names.UpToDate

        # user has propose permission
        elif utils._checkPermission(Names.ProposeContentToCollections, self):
            trusted = config.getProperty('link_policy') in Names.TrustedLinkPolicies
            if copy_obj:
                # handle updating
                if trusted:
                    auto_update = config.getProperty('auto_update')
                    # In trusted mode the collection objects content 
                    # can be savely updated (as the collection manager
                    # trusts the content objects creators).
                    action = 'update'
                    if auto_update:
                        status = verbose_status and Names.Updated or Names.UpToDate
                    else:
                        status = Names.UpdateRequestPending
                else:
                    if slot_obj.getStatus(copy_obj) in Names.AllowUntrustedUpdateStates:
                        # Object in collection can be updated savely 
                        # because it hasn't been touched by the collections
                        # manager yet.
                        action = 'update'
                    else:
                        # Object in collection got touched by the 
                        # collection manager. So do not update the
                        # collection objects content!
                        action = 'nop'
                        
                    status = Names.UpdateRequestPending
            else:
                # handle adding
                slot_obj = self._getInitialProposedSlotsObject()
                action = 'add'
                if trusted:
                    status = verbose_status and Names.Added or Names.UpToDate
                else:
                    status = Names.AddRequestPending
 
        # user has not sufficient permissions
        else:
            raise 'Unauthorized', 'No add or propose permission for %s.' % self.getId()

        return action, copy_obj, slot_obj, status

    # -----------------
    #
    # -----------------

    security.declareProtected(CMFCorePermissions.AddPortalContent, 'invokeFactory')
    def invokeFactory( self
                     , type_name
                     , id
                     , RESPONSE=None
                     , *args
                     , **kw
                     ):
        """Invokes the portal_types tool.
        
        The factory hands over the object to the appropriate slot
        factory defined by configuration.
        
        Hint:
            To get this working the 'createObject' Plone Skin Script has to
            be changed the follwoing way:
                o=context.invokeFactory(id=id, type_name=type_name)
                if o is None:
                    o = getattr(context, id, None)
        """
        ### XXX the add slot must be configurable on its own:
        ###     self._getLocalAddSlotsObject()
        slot_obj = self._getInitialAddSlotsObject()
        apply(CollectionSlot.invokeFactory,
              (slot_obj, type_name, id, RESPONSE) + args, kw)

        return slot_obj._getOb(id)

    security.declarePublic('addOrUpdateObject')
    def addOrUpdateObject(self, orig_obj):
        """Add or update an object to/in a collection.

           The object gets copied. The object carries a reference
           to the original object. Adding an object already added to
           the collection is interpreted as an update.

           Users adding an object must have the 'AddContentToCollections'
           or 'ProposeContentToCollections' permission.
        """
        # finds out what to do exactly (raises exception on
        # insufficent permissions also)
        action, copy_obj, slot_obj, status = self._getInstructionsFor(orig_obj)

        slot_obj.addOrUpdateObject(action, orig_obj, copy_obj, status)

    security.declareProtected(ModifyPortalContent, 'moveObjects')
    def moveObjects(self, ids, target_id):
        cb_copy_data = self.manage_cutObjects(ids=self._tuplyfy(ids))
        return self._pasteObjects(target_id, cb_copy_data)

    # XXX: throw away? We can decide that after having thought 
    #      about cataloging in depth!
    security.declareProtected(ModifyPortalContent, 'xxx_moveObjects')
    def xxx_moveObjects(self, ids, source_id, target_id):
        """Move objects from one slot to another
        """
        # check existence of source slot
        if getattr(aq_base(self), source_id, None):
            source_slot = getattr(self, source_id)
        else:
            raise CopyError, "Source slot '%s' does not exist" % source_id
        
        # do nothing if source and target slots are the same
        if source_id == target_id:
            return

        if getattr(aq_base(self), target_id, None):
            target_slot = getattr(self, target_id)
        else:
            raise CopyError, "Target slot '%s' does not exist" % target_id_id

        for id in ids:
            if getattr(aq_base(source_slot), id, None):
                obj = getattr(source_slot, id)
            else:
                raise CopyError, "Can't move non existing object '%s'" % id
            
            target_slot.xxx_moveObject(obj)

    # ---
    # ZMI
    # ---

    security.declareProtected(CMFCorePermissions.View, 'icon')
    def icon(self):
        """ For the ZMI """
        return self.getIcon()

    def _setObject(self, id, object, roles=None, user=None, set_owner=1):
        # only add slots to a collection, all other object types will
        # be added to the 'initial_slot_added' slot.
        # We need this if someone ads an object from the ZMI or per ftp!
        if object.meta_type == Names.collection_slot_meta_type:
            # call PortalFolders _setObject
            result = PortalFolder._setObject(self, id, object, roles, user, set_owner)
        else:
            # add object to initial add slot
            result = self._getInitialAddSlotsObject()._setObject(id, object, roles, user, set_owner)
        return result

    ### fix me: remove dummy methods to make known permissions
    def x(self): pass
    def y(self): pass
    def z(self): pass

    security.declareProtected(Names.AddContentToCollections, 'x')
    security.declareProtected(Names.ProposeContentToCollections, 'y')
    security.declareProtected(Names.ManageContentInCollections, 'z')

    # ---------------------------------------------------------
    # (Re-)Allow cataloging (PortalFolder turns cataloging off)
    # ---------------------------------------------------------

    # XXX: use SkinnedFolder instead of PortalFolder
    security.declareProtected(ModifyPortalContent, 'indexObject')
    def indexObject(self):
        """Override non cataloging behaviour of PortalFolder.
        """
        return PortalContent.indexObject(self)
    
    security.declareProtected(ModifyPortalContent, 'unindexObject')
    def unindexObject(self):
        """Override non cataloging behaviour of PortalFolder.
        """
        return PortalContent.unindexObject(self)
    
    security.declareProtected(ModifyPortalContent, 'reindexObject')
    def reindexObject(self, idxs=[]):
        """Override non cataloging behaviour of PortalFolder.
        """
        return PortalContent.reindexObject(self, idxs=idxs)

    def SearchableText(self):
        """SearchableText returns text visible by the collection view
        """
        stext = [self.title, self.description]

        """ ### fix me: catalog slots
        for o in self.objectValues():
            if self.isTitleVisible(o):
                stext.append(o.Title())
            if self.isDescriptionVisible(o):
                stext.append(o.Description())
        """

        return ' '.join(stext)

    # ------------
    # default view
    # ------------

    def _index_html(self):
        '''
        Invokes the action identified by the id "view" or the first action.
        '''
        ### fix me: choose view action depending if authentified or not ###
        tool = getToolByName(self, 'portal_types')
        ti = tool.getTypeInfo( self )
        if ti is not None:
            path = ti.getActionById('view', None)
            if path is not None:
                view = self.restrictedTraverse(path)
                return view
            actions = ti.getActions()
            if actions:
                path = actions[0]['action']
                view = self.restrictedTraverse(path)
                return view
            raise 'Not Found', ('No default view defined for type "%s"'
                                % ti.getId())
        else:
            raise 'Not Found', ('Cannot find default view for "%s"'
                                % self.getPhysicalPath())

    security.declareProtected(CMFCorePermissions.View, 'index_html')
    index_html = ComputedAttribute(_index_html, 1)

    # ---------------------
    # Visibility management
    # ---------------------

    TITLE = ObjectAnnotations.TITLE
    DESCRIPTION = ObjectAnnotations.DESCRIPTION
    BODY = ObjectAnnotations.BODY
    ALL = TITLE | DESCRIPTION | BODY
    NONE = 0
    NOT_TITLE = ALL ^ TITLE
    NOT_DESCRIPTION = ALL ^ DESCRIPTION
    NOT_BODY = ALL ^ BODY

    security.declareProtected(ModifyPortalContent, 'setTitleVisibility')
    def setTitleVisibility(self, visibility):
        """
        Sets the visibility of the title.
        """
        try:
            self.visibility = (self.visibility & self.NOT_TITLE) | (int(not(not(visibility))) * self.TITLE)
        except AttributeError:
            self.visibility = self.TITLE

    security.declareProtected(ModifyPortalContent, 'setDescriptionVisibility')
    def setDescriptionVisibility(self, visibility):
        """
        Sets the visibility of the description.
        """
        try:
            self.visibility = (self.visibility & self.NOT_DESCRIPTION) | (int(not(not(visibility))) * self.DESCRIPTION)
        except AttributeError:
            self.visibility = self.DESCRIPTION

    security.declareProtected(CMFCorePermissions.View, 'isTitleVisible')
    def isTitleVisible(self):
        """
        Returns true value if title was set to visible before.
        """
        try:
            return self.visibility & self.TITLE
        except AttributeError:
            # default for collections without visibility info is visible
            return self.TITLE

    security.declareProtected(CMFCorePermissions.View, 'isDescriptionVisible')
    def isDescriptionVisible(self):
        """
        Returns true value if description was set to visible before.
        """
        try:
            return self.visibility & self.DESCRIPTION
        except AttributeError:
            # default for collections without visibility info is visible
            return self.DESCRIPTION

    security.declarePrivate('getObjectIds')
    def getObjectIds(self, visible_only=0, groups_only=None, 
                     visible_objects_only=0):
        """Returns the collections object ids.
        
        The result set can be restricted:
           - return objects in visible slots only
           - return objects of slots of certain groups only
           - return visible objects only
        """
        ids = []
        for slot_ref in self.getSlots(visible_only=visible_only, 
                                      groups_only=groups_only):
            slot_id = slot_ref.getId()
            for obj_ref in slot_ref.listFolderContents():
                if not visible_objects_only or self.isVisible(obj_ref):
                    ids.append('%s/%s' % (slot_id, obj_ref.getId()))
                
        return ids

    security.declareProtected(CMFCorePermissions.View, 'getSlots')
    def getSlots(self, visible_only=0, groups_only=None):
        """Returns the collections slots.
        
        The result set can be restricted:
           - return visible slots only
           - return slot of certain groups
        """
        if groups_only:
            groups_only = self._tuplyfy(groups_only)
        # Slots get added to the collection by accessing them.
        # This is because there is no hook in portal_types (CMF 1.3)
        # which calls us (the collection) after beeing added.
        portal_collections = getToolByName(self, 'portal_collections')
        definition = getattr(portal_collections, Names.slots_id)
        slots_defined = {}
        for id, ref in definition.objectItems():
            slots_defined[id] = ref
        config = getattr(getattr(portal_collections, Names.types_id), 
            self.portal_type)
        slots_configured = config.objectIds()
        slots_found = self.objectIds()
        slots = []
        for slot_id in slots_configured:
            # add not yet added slots
            if slot_id not in slots_found:
                addCollectionSlot(self, slot_id)
            
            # apply filter criterias
            slot_ref = getattr(self, slot_id)
            slot_cfg_ref = slots_defined[slot_id]
            if visible_only:
                if slot_cfg_ref.getProperty('visibility') == 'hidden':
                    slot_ref = None
                if slot_cfg_ref.getProperty('visibility') == 'if not empty' \
                   and len(getattr(self, slot_id).objectIds()) == 0:
                    slot_ref = None
            if groups_only and slot_cfg_ref.getProperty('group') not in groups_only:
                slot_ref = None
            
            if slot_ref:
                slots.append(slot_ref)
        return tuple(slots)
        
    security.declareProtected(CMFCorePermissions.View, 'getVisibleObjects')
    def getVisibleObjects(self):
        """Returns the visible objects of all slots.
        
        Visible means that at least the title or the description or the 
        body has to be set visible. Returns visible objects of hidden slots
        also.
        """
        obj_refs = []
        for slot_ref in self.getSlots():
            obj_refs.extend([(slot_ref, o) for o in slot_ref.objectValues() 
                                           if slot_ref.isVisible(o)])
        return obj_refs
        
    security.declareProtected(ModifyPortalContent, 'moveRelative')
    def moveRelative(self, ids=(), positions=0):
        """Moves objects by positions (relative mode).
        """
        for id in self._tuplyfy(ids):
            slot_ref, obj_id = self._splitter(id, return_id=1)
            slot_ref.moveRelative(obj_id, positions)

    security.declareProtected(ModifyPortalContent, 'moveAbsolute')
    def moveAbsolute(self, ids=(), position=0):
        """Moves objects by positions (relative mode).
        """
        for id in self._tuplyfy(ids):
            slot_ref, obj_id = self._splitter(id, return_id=1)
            slot_ref.moveAbsolute(obj_id, position)

    ### XXX shouldn't listFolderContents, objectValues, objectIds
    ###     and objectItems call getSlots? Think about what has to 
    ###     list what!

    def _splitter(self, value, return_id=0):
        slot_id, obj_id = value.split('/')
        if getattr(aq_base(self), slot_id, None):
            slot = getattr(self, slot_id)
        else:
            raise AttributeError, \
                "no slot '%s' in collection '%s'" % (slot_id, self.getId())
        if return_id:
            return slot, obj_id
        return slot, getattr(slot, obj_id)

    def _filter(self, values, base):
        helpers = []
        for v in values:
            if v.startswith(base):
                helpers.append(v[len(base)+1:].split('/'))
        return helpers

    def _tuplyfy(self, var):
        t = type(var)
        if t in (TupleType, ListType):
            return var
        elif t in (StringType, UnicodeType):
            return (var,)
        raise TypeError, "Supply a string or a list/tuple of strings"

    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'updateVisibility')
    def updateVisibility(self, ids=(), title_vis=(), desc_vis=(), 
                         body_vis=(), helper_vis=(), coll_vis=(),
                         accept=(), reject=()):
        """Update objects visibility state
        """

        # tuplyfy
        ids = self._tuplyfy(ids)
        title_vis = self._tuplyfy(title_vis)
        desc_vis = self._tuplyfy(desc_vis)
        body_vis = self._tuplyfy(body_vis)
        helper_vis = self._tuplyfy(helper_vis)
        coll_vis = self._tuplyfy(coll_vis)
        accept = self._tuplyfy(accept)
        reject = self._tuplyfy(reject)

        # set visibility info of collection object (title and description)
        self.setTitleVisibility('Title' in coll_vis)
        self.setDescriptionVisibility('Description' in coll_vis)

        # set visibility of objects by looping over all objects
        for id in ids:
            slot, obj = self._splitter(id)
            # visibility
            slot.setTitleVisibility(obj, id in title_vis)
            slot.setDescriptionVisibility(obj, id in desc_vis)
            slot.setBodyVisibility(obj, id in body_vis)
            
            # helpers
            for (h, t) in self._filter(helper_vis, id):
                slot.setSelectedTemplate(obj, h, t)
            
            # accept/reject request
            accept_req = id in accept
            reject_req = id in reject
            if accept_req and not reject_req:
                slot.acceptPendingRequest(obj)
            elif not accept_req and reject_req:
                slot.rejectPendingRequest(obj)

# Intialize the CMFCollection content class, setting up security.
InitializeClass(CollectionType)

from Products.CMFCore.register import registerPortalContent
registerPortalContent(CollectionType,
                      constructors=(addCollectionType,),
                      action='collection_edit_form',
                      icon='collection_icon.gif')
