#########################################################################
# Copyright (c) 2002, 2003 Grgoire Weber. All Rights Reserved.
# 
# This file is part of CMFCollection.
# 
# CMFCollection is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CMFCollection is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with CMFCollection; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#########################################################################
""" CMF Collection configuration factory defaults

$Id: $
"""
__version__ = "$Revision: $"

# default slot configuration
default_slots = {
    '_properties': {
        'title':'Manages UI slots',
        'groups':['management', 'main', 'left', 'right'],
    },
    '_container': {
        'Inbox': {
            '_properties': {
                'title':'Lately added or proposed items.',
                'description':'',
                'visibility':'hidden',
                'group':'management',
                'default_text':'No items available.',
            },
        },
        'Main': {
            '_properties': {
                'title':'Main area of a collection',
                'description':'',
                'visibility':'always',
                'group':'main',
                'default_text':'No content available.',
            },
        },
        'News': {
            '_properties': {
                'title':'News',
                'description':'',
                'visibility':'if not empty',
                'group':'right',
                'default_text':'No news is good news.',
            },
        },
    },
}

# default helper configuration
default_helpers = {
    '_properties': {
        'title':'Manages UI helper skin methods',
    },
    '_container': {
        'Spacers': {
            '_properties': {
                'title':'Manage spacing between objects',
            },
            '_container': {
                'NoSpacer': {
                    '_properties': {
                        'title':'No Spacer',
                        'is_visible_info':0,
                        'template_id':'Spacer_None',
                    },
                },
                'Default': {
                    '_properties': {
                        'title':'Default Spacer',
                        'is_visible_info':0,
                        'template_id':'Spacer_Default',
                    },
                },
                'VerticalBar': {
                    '_properties': {
                        'title':'Vertical Bar',
                        'is_visible_info':0,
                        'template_id':'Spacer_VerticalBar',
                    },
                },
            },
        },
        'Footers': {
            '_properties': {
                'title':'Display additional informations at the bottom of the object',
            },
            '_container': {
                'NoFooter': {
                    '_properties': {
                        'title':'No footer',
                        'is_visible_info':0,
                        'template_id':'Footer_None',
                    },
                },
                'MoreAndLinkedTitle': {
                    '_properties': {
                        'title':'More and Linked Title',
                        'is_visible_info':1,
                        'template_id':'Footer_MoreAndLinkedTitle',
                    },
                },
                'LinkedTitle': {
                    '_properties': {
                        'title':'Linked Title',
                        'is_visible_info':1,
                        'template_id':'Footer_LinkedTitle',
                    },
                },
                'LinkedMore': {
                    '_properties': {
                        'title':'Linked More',
                        'is_visible_info':1,
                        'template_id':'Footer_LinkedMore',
                    },
                },
            },
        },
        'InfoLines': {
            '_properties': {
                'title':'Display additional informations at the top of the object',
            },
            '_container': {
                'NoInfoLine': {
                    '_properties': {
                        'title':'No Info line',
                        'is_visible_info':0,
                        'template_id':'InfoLine_None',
                    },
                },
                'DateAndLinkedCreator': {
                    '_properties': {
                        'title':'Date and Linked Creator',
                        'is_visible_info':1,
                        'template_id':'InfoLine_DateAndLinkedCreator',
                    },
                },
            },
        },
    },
}

# default types configuration
default_types = {
    '_properties': {
        'title':'Manages different CMFCollection types',
    },
    '_container': {
        'Collection': {
            '_properties': {
                'initial_slot_added':'Main',
                'initial_slot_proposed':'Inbox',
            },
            '_container': {
                'Inbox': {
                    '_properties': {
                        'default_title_visibility':1,
                        'default_description_visibility':1,
                        'default_body_visibility':0,
                        'ordering':'manual',
                        'initial_position':'top',
                    },
                    '_container': {
                    },
                },
                'Main': {
                    '_properties': {
                        'default_title_visibility':1,
                        'default_description_visibility':1,
                        'default_body_visibility':0,
                        'ordering':'manual',
                        'initial_position':'bottom',
                    },
                    '_container': {
                        'Footers': {
                            '_properties': {
                                'allowed_templates':('NoFooter', 'LinkedTitle', 'MoreAndLinkedTitle'),
                                'default_template':'MoreAndLinkedTitle',
                            },
                        },
                        'Spacers': {
                            '_properties': {
                                'allowed_templates':('Default', 'NoSpacer', 'VerticalBar'),
                                'default_template':'Default',
                            },
                        },
                    },
                },
                'News': {
                    '_properties': {
                        'default_title_visibility':1,
                        'default_description_visibility':1,
                        'default_body_visibility':0,
                        'ordering':'manual',
                        'initial_position':'top',
                    },
                    '_container': {
                        'InfoLines': {
                            '_properties': {
                                'allowed_templates':('DateAndLinkedCreator',),
                                'default_template':'DateAndLinkedCreator',
                            },
                        },
                        'Footers': {
                            '_properties': {
                                'allowed_templates':('LinkedMore',),
                                'default_template':'LinkedMore',
                            },
                        },
                        'Spacers': {
                            '_properties': {
                                'allowed_templates':('Default',),
                                'default_template':'Default',
                            },
                        },
                    },
                },
            },
        },
    },
}
