#########################################################################
# Copyright (c) 2002, 2003 Grgoire Weber. All Rights Reserved.
# 
# This file is part of CMFCollection.
# 
# CMFCollection is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# CMFCollection is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with CMFCollection; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#########################################################################
""" Manages object annotations



$Id: $
"""
__version__ = "$Revision: $"

from copy import copy

# Importing 'Persitent'makes objects persistent
# Supply a dummy class on 'ImportError'.
#try:

from Globals import Persistent
from zLOG import LOG, INFO
from Acquisition import aq_base

import Names

class ObjectAnnotations(Persistent):
    """
    Manages object annotations.
    """

    def __init__(self):
        """
        Initializes annotations.
        """
        self.setVisibility(self.NONE)
        self.setStatus(Names.JustCreated)

    def getAnnotation(self):
        """
        Returns annotation information.
        """
        return {'visibility': getattr(aq_base(self), 'visibility', None),
                'status': getattr(aq_base(self), 'status', None),
                'helpers': getattr(aq_base(self), 'helpers', None),
               }

    def setAnnotation(self, annotation):
        """
        Sets annotation information. Use results of 'getAnnotation()'.
        """
        self.visibility = annotation['visibility']
        self.status = annotation['status']
        self.helpers = annotation['helpers']


    __transitions = {
        # XXX: is this necessary?
        None: { 
            Names.UpToDate: Names.UpToDate,
            Names.Added: Names.Added,
            Names.Updated: Names.Updated,
            Names.AddRequestPending: Names.AddRequestPending,
            Names.UpdateRequestPending: Names.UpdateRequestPending,
            Names.DeleteNotificationPending: Names.DeleteNotificationPending,
            Names.OutOfDate: Names.OutOfDate,
            Names.ChangeNotificationPending: Names.ChangeNotificationPending,
            Names.IsLocal: Names.IsLocal,
            Names.JustCreated: Names.JustCreated,
        },
        Names.UpToDate: {
            Names.UpToDate: Names.UpToDate,
            Names.Added: None,
            Names.Updated: Names.Updated,
            Names.AddRequestPending: None,
            Names.UpdateRequestPending: Names.UpdateRequestPending,
            Names.DeleteNotificationPending: Names.DeleteNotificationPending,
            Names.OutOfDate: Names.UpToDate,
            Names.ChangeNotificationPending: Names.ChangeNotificationPending,
            Names.IsLocal: Names.IsLocal,
            Names.JustCreated: None,
        },
        Names.Added: {
            Names.UpToDate: Names.UpToDate,
            Names.Added: Names.Added,
            Names.Updated: Names.Added,
            Names.AddRequestPending: None,
            Names.UpdateRequestPending: Names.UpdateRequestPending,
            Names.DeleteNotificationPending: Names.DeleteNotificationPending,
            Names.OutOfDate: Names.Added,
            Names.ChangeNotificationPending: Names.ChangeNotificationPending,
            Names.IsLocal: Names.IsLocal,
            Names.JustCreated: None,
        },
        Names.Updated: {
            Names.UpToDate: Names.UpToDate,
            Names.Added: None,
            Names.Updated: Names.Updated,
            Names.AddRequestPending: None,
            Names.UpdateRequestPending: Names.UpdateRequestPending,
            Names.DeleteNotificationPending: Names.DeleteNotificationPending,
            Names.OutOfDate: Names.Updated,
            Names.ChangeNotificationPending: Names.ChangeNotificationPending,
            Names.IsLocal: Names.IsLocal,
            Names.JustCreated: None,
        },
        Names.AddRequestPending: {
            Names.UpToDate: Names.UpToDate,
            Names.Added: None,
            Names.Updated: Names.Added, # !
            Names.AddRequestPending: Names.AddRequestPending,
            Names.UpdateRequestPending: Names.AddRequestPending,
            Names.DeleteNotificationPending: Names.DeleteNotificationPending,
            Names.OutOfDate: Names.AddRequestPending,
            Names.ChangeNotificationPending: Names.AddRequestPending,
            Names.IsLocal: Names.IsLocal,
            Names.JustCreated: None,
        },
        Names.UpdateRequestPending: {
            Names.UpToDate: Names.UpToDate,
            Names.Added: None,
            Names.Updated: Names.Updated,
            Names.AddRequestPending: None,
            Names.UpdateRequestPending: Names.UpdateRequestPending,
            Names.DeleteNotificationPending: Names.DeleteNotificationPending,
            Names.OutOfDate: Names.UpdateRequestPending,
            Names.ChangeNotificationPending: Names.UpdateRequestPending,
            Names.IsLocal: Names.IsLocal,
            Names.JustCreated: None,
        },
        Names.DeleteNotificationPending: {
            Names.UpToDate: Names.IsLocal,
            Names.Added: None,
            Names.Updated: None,
            Names.AddRequestPending: None,
            Names.UpdateRequestPending: None,
            Names.DeleteNotificationPending: Names.DeleteNotificationPending,
            Names.OutOfDate: Names.UpToDate, # ! XXX: should be None ?!
            Names.ChangeNotificationPending: None,
            Names.IsLocal: Names.IsLocal,
            Names.JustCreated: None,
        },
        Names.OutOfDate: {
            Names.UpToDate: Names.UpToDate,
            Names.Added: None,
            Names.Updated: Names.Updated,
            Names.AddRequestPending: None,
            Names.UpdateRequestPending: Names.UpdateRequestPending,
            Names.DeleteNotificationPending: Names.DeleteNotificationPending,
            Names.OutOfDate: Names.OutOfDate,
            Names.ChangeNotificationPending: Names.ChangeNotificationPending,
            Names.IsLocal: Names.IsLocal,
            Names.JustCreated: None,
        },
        Names.ChangeNotificationPending: {
            Names.UpToDate: Names.UpToDate,
            Names.Added: None,
            Names.Updated: Names.Updated,
            Names.AddRequestPending: None,
            Names.UpdateRequestPending: Names.UpdateRequestPending,
            Names.DeleteNotificationPending: Names.DeleteNotificationPending,
            Names.OutOfDate: Names.OutOfDate,
            Names.ChangeNotificationPending: Names.ChangeNotificationPending,
            Names.IsLocal: Names.IsLocal,
            Names.JustCreated: None,
        },
        Names.IsLocal: {
            Names.UpToDate: Names.IsLocal, # for acceptStatus
            Names.Added: Names.Added, # addOrUpdateObject
            Names.Updated: None,
            Names.AddRequestPending: Names.AddRequestPending, # addOrUpdateObject
            Names.UpdateRequestPending: None,
            Names.DeleteNotificationPending: None,
            Names.OutOfDate: Names.IsLocal, # for rejectStatus
            Names.ChangeNotificationPending: None,
            Names.IsLocal: Names.IsLocal,
            Names.JustCreated: None,
        },
        Names.JustCreated: {
            Names.UpToDate: Names.UpToDate,
            Names.Added: Names.Added,
            Names.Updated: Names.Updated,
            Names.AddRequestPending: Names.AddRequestPending,
            Names.UpdateRequestPending: Names.UpdateRequestPending,
            Names.DeleteNotificationPending: Names.DeleteNotificationPending,
            Names.OutOfDate: Names.OutOfDate,
            Names.ChangeNotificationPending: Names.ChangeNotificationPending,
            Names.IsLocal: Names.IsLocal,
            Names.JustCreated: Names.JustCreated,
        },
    }

    __actions__ = {
        Names.UpToDate: (None, None),
        Names.Added: (Names.Acknowledge, None),
        Names.Updated: (Names.Acknowledge, None),
        Names.AddRequestPending: (Names.Accept,Names.Reject),
        Names.UpdateRequestPending: (Names.Accept,Names.Reject),
        Names.DeleteNotificationPending: (Names.Acknowledge, None),
        Names.OutOfDate: (Names.Update, None),
        Names.ChangeNotificationPending: (Names.Update, Names.Ignore),
        Names.IsLocal: (None, None),
        Names.JustCreated: (Names.Acknowledge, None),
    }

    def setStatus(self, status):
        """Set objects status.

        The resulting status depends on status to set and the status
        previously set.
        """
        prev_status = getattr(aq_base(self), 'status', None)
        status = self.__transitions[prev_status][status]

        if status is not None:
            self.status = status
        else:
            LOG('ASSERT', INFO, "Ignored impossible transition from '%s' to '%s'" % (prev_status, status))

    def getStatus(self):
        """Returns objects status.
        """
        return getattr(aq_base(self), 'status', None)

    def getActions(self):
        """Returns possible actions.
        """
        return self.__actions__[self.getStatus()]

    def acceptStatus(self):
        """Accepts pending requests and notifications.
        """
        self.setStatus(Names.UpToDate)

    def rejectStatus(self):
        """Rejects pending requests and notifications.
        """
        self.setStatus(Names.OutOfDate)


    TITLE = 1
    DESCRIPTION = 2
    BODY = 4
    ALL = TITLE | DESCRIPTION | BODY
    NONE = 0
    NOT_TITLE = ALL ^ TITLE
    NOT_DESCRIPTION = ALL ^ DESCRIPTION
    NOT_BODY = ALL ^ BODY

    def setVisibility(self, visibility=None):
        """
        Updates objects visibility state.
        """
        if visibility is None:
            self.visibility = self.TITLE | self.DESCRIPTION
        else:
            self.visibility = visibility

    def setTitleVisibility(self, visibility):
        """
        Sets the visibility of the title.
        """
        self.visibility = (self.visibility & self.NOT_TITLE) | (int(not(not(visibility))) * self.TITLE)

    def setDescriptionVisibility(self, visibility):
        """
        Sets the visibility of the description.
        """
        self.visibility = (self.visibility & self.NOT_DESCRIPTION) | (int(not(not(visibility))) * self.DESCRIPTION)

    def setBodyVisibility(self, visibility):
        """
        Sets the visibility of the body.
        """
        self.visibility = (self.visibility & self.NOT_BODY) | (int(not(not(visibility))) * self.BODY)
        
    def isTitleVisible(self):
        """
        Returns true value if title was set to visible before.
        """
        return self.visibility & self.TITLE

    def isDescriptionVisible(self):
        """
        Returns true value if description was set to visible before.
        """
        return (self.visibility & self.DESCRIPTION)

    def isBodyVisible(self):
        """
        Returns true value if body was set to visible before.
        """
        return (self.visibility & self.BODY)

    def isVisible(self):
        """
        Returns true value if content was set to visible before.
        """
        return not(not(self.visibility))

    def setTemplate(self, helper, template):
        """Sets helper to template.
        """
        if hasattr(aq_base(self), 'helpers'):
            self.helpers[helper] = template
        else:
            self.helpers = {helper: template}
        self._p_changed = 1 ### fix me: necessary?

    def getTemplate(self, helper):
        """Returns template of helper.
        """
        if hasattr(aq_base(self), 'helpers'):
            return self.helpers[helper]
        else:
            raise KeyError

    def delHelper(self, name):
        """Deletes a helper
        """
        if hasattr(aq_base(self), 'helpers'):
            del self.helpers[name]

    def getHelpers(self):
        """Returns a dictionary with helper/template pairs.
        """
        if hasattr(aq_base(self), 'helpers'):
            return copy(self.helpers)

    def setHelpers(self, helpers):
        """Sets Helpers.
        """
        self.helpers = helpers
        self._p_changed = 1 ### fix me: necessary?

    def delHelpers(self):
        """Sets Helpers.
        """
        if hasattr(aq_base(self), 'helpers'):
            del self.helpers

